package no.ingeberg.NewsML;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * AdminMetadata.java <br>
 * Klassen innholder informasjon som beskriver opprinnelsen til en nyhetskomponent,
 * og har metoder for  sette ulike element og en metode for  hente ut disse 
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */
public class AdminMetadata {
    private Catalog catalog         = null;// 0-1
    private File fileName           = null;// 0-1
    private URL systemIdentifier    = null;// 0-1
    private String provider         = null;// 0-1
    private String creator          = null;// 0-1
    private int sourceNr            = 0; //source er en URL som kan forekomme 1-N ganger.
    private ArrayList source        = null;
    private int contributorNr       = 0; // contributor er en string som kan forekomme 1-N ganger.
    private ArrayList contributor   = null;
    private int properyNr           = 0; //property er en string som kan forekomme 1-N ganger.
    private ArrayList property      = null;
    
    /**
     *Oppretter en ny istans av AdminMetadata, i denne konstruktoren er alle under elementene tatt med.
     *<p>
     *@param catalog    Identifiserer standard vokabular for AdminMetadata. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param fileName   Identifiserer anbefalte eller faktiske filnavn(path) for nyhetsobjektet. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param systemIdentifier   URL som identifiserer system addressen<br>(som en URL) hvor posten er.<br>
     *                          Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param provider   String som identifiserer hvem som har utgitt nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param creator    String som identifiserer hvem som har laget nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param source     URL som identifiserer kilden som har tilfrt kilde materialet til nyhets objektet. <br>
     *                  Her kan en ogs legge inn en NewsItem attributt som innholder en URN til en nyhetspost, <br>
     *                  hvis kilden har en egen nyhetspost.<br>
     *                  Elementet er frivillig og repiterbart.<br><br>
     *@param contributor  En string som identifiserer hvem som har modifisert eller forbedret nyhetsobjektet etter at den ble laget. <br>
     *                    Her kan en ogs legge inn et kommentarfelt om en nsker det.<br>
     *                    Elementet er frivillig og repiterbart.<br><br>
     *@param property     En string som kan brukes til  beskrive vrig administrative metadata som ikke blir dekket av de vrige.<br>
     *                    Elementet er frivillig og repiterbart.<br><br>
     *
     */
    public AdminMetadata(Catalog catalog, File fileName, URL systemIdentifier,
    String provider, String creator, String source, String contributor, String property){
        this.catalog           = catalog;
        this.fileName          = fileName;
        this.systemIdentifier  = systemIdentifier;
        this.provider          = provider;
        this.creator           = creator;
        this.source            = new ArrayList();
        this.contributor       = new ArrayList();
        this.property          = new ArrayList();
        this.source.add(source);
        this.contributor.add(contributor);
        this.property.add(property);
    }
    
    /**
     *Oppretter en ny istans av AdminMetadata, i denne konstruktoren er flgende underelementene tatt med.
     *<p>
     
     *@param provider   String som identifiserer hvem som har utgitt nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param creator    String som identifiserer hvem som har laget nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *@param source     URL som identifiserer kilden som har tilfrt kilde materialet til nyhets objektet. <br>
     *                  Her kan en ogs legge inn en NewsItem attributt som innholder en URN til en nyhetspost, <br>
     *                  hvis kilden har en egen nyhetspost.<br>
     *                  Elementet er frivillig og repiterbart.<br><br>
     */
    public AdminMetadata(String creator, String source){
        this.creator           = creator;
        this.source            = new ArrayList();        
        this.source.add(source);
    }
    
    /**Oppretter et tom AdminMetadata objekt*/
    public AdminMetadata(){
        //Det er lov  opprette tomme objekt.
    }
    
    /**
     * Metoden setter catalog element.
     *  @param catalog  Identifiserer standard vokabular for AdminMetadata. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setCatalog(Catalog catalog){
        this.catalog = catalog;
    }
    
    /**
     *Metoden setter fileName elementet
     *@param fileName   Identifiserer anbefalte eller faktiske filnavn(path) for nyhetsobjektet. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setFileName(File fileName){
        this.fileName = fileName;
    }
    
    /**
     *Metoden setter systemIdentifier elementet.
     *@param systemIdentifier   URL som identifiserer system addressen (som en URL) hvor posten er.
     *                          Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     *
     */
    public void setSystemIdentifier(URL systemIdentifier){
        this.systemIdentifier  = systemIdentifier;
    }
    
    /**
     *Metoden setter provider elementet.
     *@param provider   String som identifiserer hvem som har utgitt nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setProvider(String provider){
        this.provider = provider;
    }
    
    /** Metoden setter creator elementet.
     * @param creator    String som identifiserer hvem som har laget nyhetsobjektet, <br>
     *                  kan vre et individ og/eller en bedrift og/eller en organisasjon.<br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setCreator(String creator){
        this.creator = creator;
    }
    
    /**Metoden legger til "ekstra" source element. Kan gjentaes s ofte som nskelig.
     *@param source     URL som identifiserer kilden som har tilfrt kilde materialet til nyhets objektet. <br>
     *                  Her kan en ogs legge inn en NewsItem attributt som innholder en URN til en nyhetspost, <br>
     *                  hvis kilden har en egen nyhetspost.<br>
     */
    public void addSource(String source){
        this.source.add(source);
    }
    
    /**Metoden legger til "ekstra" contributor element. Kan gjentaes s ofte som nskelig.
     * @param contributor  En string som identifiserer hvem som har modifisert eller forbedret nyhetsobjektet etter at den ble laget.
     *                    Her kan en ogs legge inn et kommentarfelt om en nsker det.<br>
     */
    public void addContributor(String contributor){
        this.contributor.add(contributor);
    }
    
    /**Metoden legger til "ekstra" property element. Kan gjentaes s ofte som nskelig.
     *@param property     En string som kan brukes til  beskrive vrig administrative metadata som ikke blir dekket av de vrige.
     *                    Elementet er frivillig og repiterbart.
     */
    public void addProperty(String property){
        this.property.add(property);
    }
    
    /**
     * Metoden returnerer alle AdminMetadata variablene, i xml-format.
     */
    public String getAdminMetadata(){
                /*<AdministrativeMetadata>
                        <Catalog Href="Som beskrevet over."/>
                        <FileName>Frivillig element som identifiserer anbefalte eller faktiske filnavn(path) for nyhetsobjektet.</FileName>
                        <SystemIdentifier>Frivillig elementet som identifiserer system addressen (som en URL) hvor posten er.</SystemIdentifier>
                        <Provider>
                                <Party FormalName="Frivillig element som identifiserer hvem som har utgitt nyhetsobjektet, kan vre et individ 
                                    og/eller en bedrift og/eller en organisasjon. Her kan en ogs legge inn et kommentarfelt om en nsker det."/>
                        </Provider>
                        <Creator>
                                <Party FormalName="Frivillig element som identifiserer hvem som har laget nyhetsobjektet, kan vre et individ 
                                    og/eller en bedrift og/eller en organisasjon. Her kan en ogs legge inn et kommentarfelt om en nsker det."/>
                        </Creator>
                        <Source>
                                <Party FormalName="Frivillig og repeterbart element som identifiserer kilden som har tilfrt kilde materialet til 
                                    nyhetsobjektet. Her kan en ogs legge inn en NewsItem attributt som innholder en URN til en nyhetspost, hvis 
                                    kilden har en egen nyhetspost."/>
                        </Source>
                        <Contributor>
                            <Party FormalName="Frivillig og repeterbart element som identifiserer hvem som har modifisert eller forbedret 
                                nyhetsobjektet etter at den ble laget. Her kan en ogs legge inn et kommentarfelt om en nsker det."/>
                        </Contributor>
                        <Property FormalName="og repeterbart element som kan brukes til  beskrive vrig administrative metadata som ikke blir 
                            dekket av de vrige."/>
                </AdministrativeMetadata>
                 */
        String t = "<AdministrativeMetadata>";
        if (catalog !=null) t= t + catalog.getCatalog();
        if (fileName != null) {t= t + "<FileName>" + fileName + "\"</FileName>"; }
        if (systemIdentifier != null){t= t + "<SystemIdentifier>" + systemIdentifier +"\"</SystemIdentifier>";}
        if (provider != null) {
            t= t + "<Provider>";
            t= t + "<Party FormalName=\"" + provider + "\"/>";
            t= t + "</Provider>";
        }
        if (creator != null) {
            t += "<Creator>";
            t += "<Party FormalName=\""+ creator + "\"/>";
            t += "</Creator>";
        }
        
        if ( source != null) {
            t= t + "<Source>";
            Iterator i = source.iterator();
            while (i.hasNext()){
                t = t + "<Party FormalName=\"" + ((String)i.next()).toString() + "\"/>";
            }
            t= t + "</Source>";
        }
        if (contributor!= null) {
            t= t + "<Contributor>";
            Iterator i = source.iterator();
            while (i.hasNext()){
                t = t + "<Party FormalName=\"" + ((String)i.next()).toString() + "\"/>";
            }
            t= t + "</Contributor>";
        }
        if (property!= null) {
            Iterator i = source.iterator();
            while (i.hasNext()){
                t = t + "<Property FormalName=\"" + ((String)i.next()).toString() + "\"/>";
            }
        }
        t= t + "</AdministrativeMetadata>";
        return t;
    }
}