package no.ingeberg.NewsML;
import java.net.*;
import java.util.*;

/**
 * Catalog.java <br>
 * Klassen hndterer innkomne data under "Catalog" elementet, som identifiserer eller lokaliserer standard
 * vokabular og indikerer hvor i NewsML dokumentet visse Topics er brukt. Catalog elementet tillatter 
 * bestemme URN, URL og fastsette hvilke vokabular (TopicSet) som er standard for elementet typen
 * i en gitt kontekst.
 * Klassen innholder metoder for  hente ut disse formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */


public class Catalog {    
    private String catalogHref      = null; // 0-1
    private int resourceNr          = 0;  //Resource er en string som kan ha 0-N forekomster.
    private ArrayList resource      = null;
    private int topicUseNr          = 0; 
    //TopicUse er en kombianasjon av to stringer, topicUse og Context, som forekommer parvis, parrat kan ha 0-N forekomster. 
    // Denne stringen skal vre en peker til et topic som alt finnes!
    private ArrayList topicUseMap   = null;
    
    /**
     *Oppretter en ny istans av catalog, i denne constructoren er alle under elementene tatt med.
     * @param CatalogHref   Angir en peker til et annet catalog element i dette eller i andre newsML dokument. <br><br>
     * @param resource      Identifiserer en ekstern ressurs, og den aktuelle ressursen opptrer som en standard vokabular for deler 
     *                          eller hele elementets innhold.<br><br>
     * @param topicUse      Danner sammen med stringTopic elementet TopicUse som indikerer hvor i NewsML dokumentet spesifikke emner er 
     *                      brukt i dokumentet. Verdien til dette elementet er en peker som innholder en #karakter fulgt av verdien til 
     *                      et "Duid" attributt for et emne i det aktuelle dokumentet. <br><br>
     * @param stringTopic   Danner sammen med topicUse elementet TopicUse som indikerer hvor i NewsML dokumentet spesifikke emner er brukt 
     *                      i dokumentet.  Verdien til "context" attributtet er et XPath mnster som indikerer konteksten der emnet 
     *                      er bruk, innen subbtreet som aktuelle "Catalog" fastsetter. Er ikke "context" elementet brukt, vil 
     *                      "TopicUse" elementet kun stadfeste hvilke emner som er tilstede i subtreet.<br><br>
     */
    public Catalog(String CatalogHref, Resource resource, String topicUse, String stringTopic){
        this.catalogHref        = catalogHref;
        this.resource           = new ArrayList();
        this.resource.add(resource);
        this.topicUseMap        = new ArrayList();
        addTopicUseMap(topicUse,stringTopic);
    }
    
    
    /** Minste lovlige constructor, skal catalog-elementet finnes m den innholde minst en "resourse"
     * @param resource      Identifiserer en ekstern ressurs, og den aktuelle ressursen opptrer som en standard vokabular for deler eller hele 
     *                      elementets innhold.<br><br>
     */
    public Catalog(Resource resource){
        this.resource = new ArrayList();
        this.resource.add(resource);
    }
    
    /**Metoden legger til "ekstra" resource element. Kan gjentaes s ofte som nskelig.
     * @param resource      Identifiserer en ekstern ressurs, og den aktuelle ressursen opptrer som en standard vokabular for deler eller hele 
     *                      elementets innhold.<br><br>
     */
    public void addResousce(Resource resource){
        this.resource.add(resource);
    }
    
    /**Metoden legger til "ekstra" topicUse element. Kan gjentaes s ofte som nskelig.
     * @param topicUse      Danner sammen med stringTopic elementet TopicUse som indikerer hvor i NewsML dokumentet spesifikke emner er brukt i 
     *                      dokumentet. Verdien til dette elementet er en peker som innholder en #karakter fulgt av verdien til et "Duid" 
     *                      attributt for et emne i det aktuelle dokumentet. <br><br>
     * @param stringTopic   Danner sammen med topicUse elementet TopicUse som indikerer hvor i NewsML dokumentet spesifikke emner er brukt i 
     *                      dokumentet.  Verdien til "context" attributtet er et XPath mnster som indikerer konteksten der emnet er bruk,
     *                      innen subbtreet som aktuelle "Catalog" fastsetter. Er ikke "context" elementet brukt, vil "TopicUse"
     *                      elementet kun stadfeste hvilke emner som er tilstede i subtreet.<br><br>
     */
    public void addTopicUseMap(String topicUse, String stringTopic){
        this.topicUseMap.add(new TopicUseMap(topicUse,stringTopic));
    }
    
    /**
     * Metoden returnerer alle catalog variablene, i xml-format.
     *
     */
    public String getCatalog() {
        String t = null;
        if (catalogHref != null) {
            t = "<Catalog Href=\""+ catalogHref + ">";
        }
        else t = "<Catalog>";
        
        if ( resource != null) {
            Iterator i = resource.iterator();
            Object o;
            while (i.hasNext()){
                //Skriver ut alle elementene som ligger inne i arrayListen
                t = t + ((no.ingeberg.NewsML.Resource)i.next()).getResource();
            }
        }
        
        if (topicUseMap != null){
            Iterator i = topicUseMap.iterator();
            while (i.hasNext()){
                t = t + i.next().toString();
            }
        }
        t = t + "</Catalog>";
        return t;
    }
    
    /////////////////////////////////////////////////
    //////////////// PRIVATE CLASS //////////////////
    /////////////////////////////////////////////////
    /*Metoden hndterer innkomne data til TopicUse elementet, som indikerer hvor i NewsML dokumentet spesifikke emner er brukt i dokumentet.
     @param topicUse        Verdien til dette elementet er en peker som innholder en #karakter fulgt av verdien til et "Duid" attributt for 
                            et emne i det aktuelle dokumentet. <br><br>
     * @param stringTopic   Verdien til "context" attributtet er et XPath mnster som indikerer konteksten der emnet er bruk, innen 
                            subbtreet som aktuelle "Catalog" fastsetter. Er ikke "context" elementet brukt, vil "TopicUse" 
                            elementet kun stadfeste hvilke emner som er tilstede i subtreet.<br><br>
     */
    
    private class TopicUseMap {
        String topicUse = null;
        String stringTopic = null;
        
        public TopicUseMap(String topicUse, String stringTopic) {
            this.topicUse       = topicUse;
            this.stringTopic    = stringTopic;
        }
        
        public String toString() {
            return new String("<TopicUse Topic=\"" + topicUse + "\" Context=\""  + stringTopic + "\"/>");
        }
    }
}
