package no.ingeberg.NewsML;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * DescriptiveMetadata.java<br>
 * Klassen innholder informasjon som beskriver innholdet til en nyhetskomponent
 * og har metoder for  sette ulike element og en metode for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class DescriptiveMetadata {
    private Catalog catalog         = null; // 0-1
    private int languageNr          = 0; // Language er en string som kan forekomme 0-N ganger. Verdiene hentes fra kont. vokabular.
    private ArrayList language      = null;
    private String genre            = null;// 0-1 Verdiene hentes fra kont. vokabula
    private int subjectCodeNr       = 0; 
    //subjectCode er en string som kan forekomme 0-N ganger. Verdiene hentes fra kont. vokabular, IPTC's Subject Codes
    private ArrayList subjectCode   = null;
    private int ofInterestToNr      = 0;  //ofInterestTo er en string som kan forekomme 0-N ganger. Verdiene hentes fra kont. vokabular.
    private ArrayList ofInterestTo  = null;
    private ArrayList topicOccurMap = null;
    private int propertyNr          = 0; //property er en string som kan forekomme 0-N ganger.
    private ArrayList property      = null;
    
    /**
     *  Konstruktoren oppretter et komplett objekt med alle lovelege elementer.
     *  <p>
     *  @param catalog  Identifiserer standard vokabular for DescriptionMetadata. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *  @param language En string som angir sprket p innholdet til nyhetsposten.<br>
     *                  Verdiene hentes fra et kontrollert vokabular. <br>
     *                  Elementet er frivillig og repeterbart.<br><br>
     *  @param genre    En string som angir hvilken nyhets-sjanger innholdet til nyhetsposten tilhrer. <br>
     *                  Verdiene hentes fra et kontrollert vokabular. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     *  @param subjectCode  En sting som angir emnet til nyhetsposten.  <br>
     *                      Verdiene hentets fra IPTC subject Codes.  <br>
     *                      Elementet er frivillig og repeterbart. <br> <br>
     *  @param ofInterestTo En string som angir mlgruppen for nyhetsposten. <br>
     *                      Verdiene her er hentet fra et kontrollert vokabular.  <br>
     *                      Elementet er frivillig og repeterbart. <br> <br>
     *  @param topicOccurrenceTopic En URL som sammen med topicOccurrence gir topicOccurrence elementet,
     *                              som angir emnet til nyhetskomponenten.  <br>
     *                              Denne parameteren gir topic attributt til elementet som angir elementetes Duid. <br>
     *                              Elementet er frivillig og repeterbart. <br> <br>
     *  @param topicOccurrence  En string som sammen topicOccurrenceTopic gir topicOccurrence elementet,
     *                          som angir emnet til nyhetskomponenten.  <br>
     *                          Denne parameteren gir howPresent attributt som indikerer naturen til emnet. <br>
     *                          Elementet er frivillig og repeterbart. <br> <br>
     *  @param property     En string som kan brukes til  beskrive vrig administrative metadata som ikke blir dekket av de vrige.  <br>
     *                      Elementet er frivillig og repeterbart. <br> <br>
     *  <p>
     *  @see #setCatalog(Catalog catalog)
     */
    public DescriptiveMetadata(Catalog catalog, String language, String genre,
    String subjectCode, String ofInterestTo, URL topicOccurrenceTopic, String topicOccurrence,
    String property){
        this.catalog                = catalog;
        this.language               = new ArrayList();
        this.genre                  = genre;
        this.subjectCode            = new ArrayList();
        this.ofInterestTo           = new ArrayList();
        this.property               = new ArrayList();
        this.language.add(language);
        this.subjectCode.add(subjectCode);
        this.ofInterestTo.add(ofInterestTo);
        this.property.add(property);
        addTopicOccurMap(topicOccurrenceTopic, topicOccurrence);
    }
    
    /** Denne konstruktoren oppretter en ny, -tom instans av DescriptionMetadata.*/
    public DescriptiveMetadata(){
        //Det er lov  opprette et tomt descriptiveMetadata objekt.
    }
    
    /**
     * Metoden setter catalog element.
     *  @param catalog  Identifiserer standard vokabular for DescriptionMetadata. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setCatalog(Catalog catalog){
        this.catalog = catalog;
    }
    
    /** Metoden setter "genre" elementet.
     *  @param genre    En string som angir hvilken nyhets-sjanger innholdet til nyhetsposten tilhrer. <br>
     *                  Verdiene hentes fra et kontrollert vokabular. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setGenre(String genre){
        this.genre = genre;
    }
    
    /**Metoden legger til "ekstra" language element.
     *  @param language En string som angir sprket p innholdet til nyhetsposten.<br>
     *                  Verdiene hentes fra et kontrollert vokabular. <br>
     *                  Elementet er frivillig og repeterbart.<br>
     */
    public void addLanguage(String language){
        if (this.language ==null) this.language = new ArrayList();
        this.language.add(language);
    }
    
    /**Metoden legger til "ekstra" subjectCode element.
     *  @param subjectCode  En sting som angir emnet til nyhetsposten.  <br>
     *                      Verdiene hentets fra IPTC subject Codes.  <br>
     *                      Elementet er frivillig og repeterbart. <br>
     */
    public void addSubjectCode(String subjectCode){
        this.subjectCode.add(subjectCode);
    }
    
    /**Metoden legger til "ekstra" ofInterestTo  element.
     *  @param ofInterestTo En string som angir mlgruppen for nyhetsposten. <br>
     *                      Verdiene her er hentet fra et kontrollert vokabular.  <br>
     *                      Elementet er frivillig og repeterbart. <br>
     */
    public void addOfInterestTo(String ofInterestTo){
        this.ofInterestTo.add(ofInterestTo);
    }
    
    /**Metoden legger til "ekstra" property element.
     *  @param property     En string som kan brukes til  beskrive vrig administrative metadata som ikke blir dekket av de vrige.  <br>
     *                      Elementet er frivillig og repeterbart. <br>
     */
    public void addProperty(String property){
        this.property.add(property);
    }
    
    /**Metoden legger til "ekstra" topicOccurens element, med bde topic og howpresent som parameretere. Kan gjentaes s ofte som nskelig.
     *  @param topicOccurrenceTopic En URL som sammen med topicOccurrence gir topicOccurrence elementet,
     *                              som angir emnet til nyhetskomponenten.  <br>
     *                              Denne parameteren gir topic attributt til elementet som angir elementetes Duid. <br>
     *                              Elementet er frivillig og repeterbart. <br> <br>
     *  @param topicOccurrence  En string som sammen topicOccurrenceTopic gir topicOccurrence elementet,
     *                          som angir emnet til nyhetskomponenten.  <br>
     *                          Denne parameteren gir howPresent attributt som indikerer naturen til emnet. <br>
     *                          Elementet er frivillig og repeterbart. <br>
     */
    public void addTopicOccurMap(URL topicOccurrenceTopic, String topicOccurrence){
        if (topicOccurMap == null) this.topicOccurMap = new ArrayList();
        this.topicOccurMap.add(new TopicOccurMap(topicOccurrenceTopic, topicOccurrence));
    }
    
    /** Metoden legger til "ekstra" topicOccurens element, med bare topic som paramereter. Kan gjentaes s ofte som nskelig.
     *  @param topicOccurrenceTopic En URL som sammen med topicOccurrence gir topicOccurrence elementet,
     *                              som angir emnet til nyhetskomponenten.  <br>
     *                              Denne parameteren gir topic attributt til elementet som angir elementetes Duid. <br>
     *                              Elementet er frivillig og repeterbart. <br> <br>
     */
    public void addTopicOccurMap(URL topicOccurrenceTopic){
        if (topicOccurMap == null) this.topicOccurMap = new ArrayList();
        this.topicOccurMap.add(new TopicOccurMap(topicOccurrenceTopic));
    }
    
    /**
     * Metoden returnerer alle DescriptiveMetadata variablene, i xml-format.
     *
     */
    public String getDescriptiveMetadata(){
        /*<DescriptiveMetadata>
                <Catalog Href="som beskrevet over."/>
                <Language FormalName="Frivillig og repeterbart element som angir sprket p innholdet til nyhetsposten. Verdiene hentes fra et 
                            kontrollert vokabular."/>
                <Genre FormalName="hvilken sjanger p innholdet til nyhetsposten tilhrer. Verdiene hentes fra et kontrollert vokabular."/>
                <SubjectCode AssignedBy="Angir emnet til nyhetsposten. Verdiene hentets fra IPTC subject Codes."/>
                <OfInterestTo FormalName="Mlgruppen for nyhetsposten. Har underelementet Relevance. Verdiene her er hentet fra et kontrollert 
                                vokabular."/>
                <TopicOccurrence HowPresent="Angir emnet til nyhetskomponenten. Elementet har et howPresent attributt som indikerer naturen til 
                                    emnet, og et emne attribut som angir en Duid."/>
                <Property FormalName="Frivillig og repeterbart element som kan brukes til  beskrive vrig administrative metadata som ikke blir
                            dekket av de vrige."/>
        </DescriptiveMetadata>
         */
        String t = "<DescriptiveMetadata>";
        if (catalog != null){t = t + catalog.getCatalog();}
        if (language!= null){
            Iterator i = language.iterator();
            while (i.hasNext()){
                t = t +"<Language FormalName=\"" + i.next().toString() + "\"/>";
            }
        }
        
        if (genre != null){t = t + "<Genre FormalName=\"" + genre + "\"/>";}
        if (subjectCode !=null){
            Iterator i = subjectCode.iterator();
            while (i.hasNext()){
                t = t +"<SubjectCode AssignedBy=\"" + ((String)i.next()).toString() + "\"/>";
            }
        }
        if ( ofInterestTo != null){
            Iterator i = ofInterestTo.iterator();
            while (i.hasNext()){
                t = t +"<OfInterestTo FormalName=\"" + ((String)i.next()).toString() + "\"/>";
            }
        }
        if (topicOccurMap != null){
            Iterator i = topicOccurMap.iterator();
            while (i.hasNext()) i.next().toString();
        }
        
        if (property != null){
            Iterator i = ofInterestTo.iterator();
            while (i.hasNext()){
                t = t +"<Property FormalName=\"" + ((String)i.next()).toString() + "\"/>";
            }
        }
        t = t + "</DescriptiveMetadata>";
        return t;
    }
    
    /////////////////////////////////////////////////
    //////////////// PRIVATE CLASS TopicOccurMap//////
    /////////////////////////////////////////////////
    /**Klassen legger til "ekstra" topicOccurens element, med bde topic og howpresent som parameretere.*/
    
    private class TopicOccurMap {
        private String topicOccurrence  = null;// 0-N
        private URL topicOccurrenceTopic= null;// som topicOccurence
        
        /**Metoden legger til "ekstra" topicOccurens element, med bde topic og howpresent som parameretere. Kan gjentaes s ofte som nskelig.
         *  @param topicOccurrenceTopic En URL som sammen med topicOccurrence gir topicOccurrence elementet,
         *                              som angir emnet til nyhetskomponenten.  <br>
         *                              Denne parameteren gir topic attributt til elementet som angir elementetes Duid. <br>
         *                              Elementet er frivillig og repeterbart. <br> <br>
         *  @param topicOccurrence  En string som sammen topicOccurrenceTopic gir topicOccurrence elementet,
         *                          som angir emnet til nyhetskomponenten.  <br>
         *                          Denne parameteren gir howPresent attributt som indikerer naturen til emnet. <br>
         *                          Elementet er frivillig og repeterbart. <br>
         */
        public TopicOccurMap(URL topicOccurrenceTopic, String topicOccurrence) {
            this.topicOccurrence       = topicOccurrence;
            this.topicOccurrenceTopic  = topicOccurrenceTopic;
        }
        
        /**Metoden legger til "ekstra" topicOccurens element, med bde topic og howpresent som parameretere. Kan gjentaes s ofte som nskelig.
         *  @param topicOccurrenceTopic En URL som sammen med topicOccurrence gir topicOccurrence elementet,
         *                              som angir emnet til nyhetskomponenten.  <br>
         *                              Denne parameteren gir topic attributt til elementet som angir elementetes Duid. <br>
         *                              Elementet er frivillig og repeterbart. <br> <br>
         */
        public TopicOccurMap(URL topicOccurrenceTopic) {
            this.topicOccurrenceTopic  = topicOccurrenceTopic;
        }
        
        /**Metoden retunerer topicOccurrens elemetnet formatert som xml*/
        public String toString() {
            String t =  new String("<TopicOccurrence Topic=\"" +  topicOccurrenceTopic + "\"");
            if (topicOccurrence != null) t = t + " HowPresent=\"" + topicOccurrence + "\" ";
            t = t +"/>";
            return t;
        }
    }
}