package no.ingeberg.NewsML;
import java.net.*;
import java.util.*;
/**
 * NewsComponent.java<br>
 * Klassen hndterer innkomne data under "NewsComponent" elementet, som er et beholder objekt 
 * som grupperer sammen ulike nyhetsobjekt (nyhetsposter, nyhetspost referanser, og nyhetskomponenter) og innholds element.  
 * Klassen jobber mot sub-klassen NewsComponent class, som tar seg av hndteringa av
 * NewsItem | NewsItemRef | NewsComponent | ContentItem elementene der kun en
 * var de skal forekomme. Klassen innholder og metoder for  hente ut disse
 * formatert som xml, etter NewsML DTD'en.
 *
 * @author Marit Ingeberg
 */

public class NewsComponent {
    private String essential                        = null; //0-1 attributt, skal innholde yes/no
    private String equivalentsList                  = null; //0-1 attributt, skal innholde yes/no
    //EquivalentsList attribute indicates whether or not the NewsItems or NewsItemRefs, NewsComponents orContentItems contained 
    //within this one are equivalent to one another in content and/or meaning.
    private String xmlLang                          = null; //0-1 attributt
    private ArrayList comment                       = null; //0-N
    private Catalog catalog                         = null; //0-1
    private ArrayList topicSet                      = null; //0-N
    private String role                             = null; //0-N
    private int basisForChoiceNr                    = 0; //basisForChoice er en string som kan forekomme 0-N ganger
    private ArrayList basisForChoice                = null;
    private NewsLines newsLines                     = null; //0-1
    private AdminMetadata administrativeMetadata    = null; //0-1
    private RightsMetadata rightsMetadata           = null; //0-1
    private DescriptiveMetadata descriptiveMetadata = null; //0-1
    private String metadata                         = null; //0-N
    private String metadataProperty                 = null; //som metadata
    private NewsComponentSub newsComponentSub       = null; //0-1
    
    
    /**
     *Oppretter en ny istans av NewsComponent, i denne konstruktoren er alle
     * under-elementene tatt med
     * @param essential                Indikerer om utgiver anser denne nyhetskomponentet til  vre av essensiell i forhold til konteksten der 
     *                                 den str. Her kan en velge mellom "yes" og "no". "No" er satt som default.<br><br>
     * @param equivalentsList          Indikerer om de elementene som nyhetskomponenten bestr av er alternative objekter. Slik som nr en 
     *                                 artikkel er tilgjenglig i ulike typer sprk. Her kan en velge mellom "yes" og "no". "No" er 
     *                                 satt som default.<br><br>
     * @param xmlLang                  Angir sprket i nyhetskomponenten. M angis i IETF RFC 3066.<br><br>
     * @param comment                  Kommentar element.<br><br>
     * @param catalog                  Identifiserer standard vokabular for newsComponent. <br>
     *                                 Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                                 Elementet er frivillig, og kan kun forekomme 1 gang.<br><br>
     * @param topicSet                 Beskriver emnet til nyhetskomponentet. <br><br>
     * @param role                     Spesifiserer rollen en nyhetskomponent spiller i den nyhetskomponenten den er den del av. <br><br>
     * @param basisForChoice           Hvis det finnes flere alternative elementer i attributtetet "equivalentsList", angir en de  
     *                                 komponentene her. Dette elementet bruker et Xpath mnster, eller en element-type navn for 
     *                                 identifisere den rette komponenten i forhold til de egenskapene som er gitt.<br><br>
     * @param newsLines                Angir karakteristiske egenskaper ved nyhetsobjektet. Disse er ment  tilfre en representasjon av 
     *                                 metadata som er lett tilgjenglig for mennesker.<br><br>
     * @param administrativeMetadata   Angir metadata som forteller om opprinnelsen til nyhetskomponenten.<br><br>
     * @param rightsMetadata           Angir metadata innholder informasjon om opphavsrett, og rett til bruk vedrrende nyhetskomponentene.
     *                                 <br><br>
     * @param descriptiveMetadata      Angir metadata innholder informasjon som beskriver innholdet til en nyhetskomponent.<br><br>
     * @param metadata                 Angir metadata som ikke er dekket av de vrige. Gir metadata type navn.<br><br>
     * @param metadataProperty         Angir metadata som ikke er dekket av de vrige. Gir metadata verdi.<br><br>
     * @param newsComponentSub         Kaller et av elementene: NewsItem | NewsItemRef | NewsComponent | ContentItem  der kun et av de skal 
     *                                 forekomme.<br><br>
     *
     */
    public NewsComponent(String essential, String equivalentsList,
    String xmlLang,String comment, Catalog catalog, TopicSet topicSet,
    String role, String basisForChoice, NewsLines newsLines, AdminMetadata administrativeMetadata,
    RightsMetadata rightsMetadata, DescriptiveMetadata descriptiveMetadata, String metadata, String metadataProperty,
    NewsComponentSub newsComponentSub){
        this.essential                 = essential;
        this.equivalentsList           = equivalentsList;
        this.xmlLang                   = xmlLang;
        this.comment                   = new ArrayList();
        this.catalog                   = catalog;
        this.topicSet                  = new ArrayList();
        this.role                      = role;
        this.basisForChoice            = new ArrayList();
        this.newsLines                 = newsLines;
        this.administrativeMetadata    = administrativeMetadata;
        this.rightsMetadata            = rightsMetadata;
        this.descriptiveMetadata       = descriptiveMetadata;
        this.metadata                  = metadata;
        this.metadataProperty          = metadataProperty;
        this.newsComponentSub          = newsComponentSub;
        this.comment.add(comment);
        this.topicSet.add(topicSet);
        this.basisForChoice.add(basisForChoice);
        
    }
    
    /**
     *Oppretter en ny istans av NewsComponent, i denne konstruktoren flgende under-elementene tatt med     
     * @param comment                  Kommentar element.<br><br>
     * @param newsLines                Angir karakteristiske egenskaper ved nyhetsobjektet. Disse er ment  tilfre en representasjon av 
     *                                 metadata som er lett tilgjenglig for mennesker.<br><br>
     * @param administrativeMetadata   Angir metadata som forteller om opprinnelsen til nyhetskomponenten.<br><br>
     * @param newsComponentSub         Kaller et av elementene: NewsItem | NewsItemRef | NewsComponent | ContentItem  der kun et av de skal 
     *                                 forekomme.<br><br>
     *
     */
    public NewsComponent(String comment, NewsLines newsLines, AdminMetadata administrativeMetadata, NewsComponentSub newsComponentSub){        
        this.comment                   = new ArrayList();        
        this.newsLines                 = newsLines;
        this.administrativeMetadata    = administrativeMetadata;       
        this.newsComponentSub          = newsComponentSub;
        this.comment.add(comment);
        
    }
    
    public NewsComponent(){
        //det er lov  opprette et tomt NewsComponent objekt.
    }
    
    /**Metoden setter essential attributtet.
     * @param essential Indikerer om utgiver anser denne nyhetskomponentet til  av essensiell i forhold til konteksten der den str.
     * Her kan en velge mellom "yes" og "no". "No" er satt som default.<br><br>
     */
    public void setEssential(String essential){
        this.essential = essential;
    }
    
    /**Metoden setter essential attributtet.
     * @param equivalentsList Indikerer om de elementene som nyhetskomponenten bestr av er alternative objekter. 
     *                         Slik som nr en artikkel er tilgjenglig i ulike typer sprk. Her kan en velge mellom "yes" og "no".
     *                          "No" er satt som default.
     */
    public void setEquivalentsList(String equivalentsList){
        this.equivalentsList  = equivalentsList;
    }
    
    /**Metoden setter essential attributtet.
     * @param xmlLang Angir sprket i nyhetskomponenten. M angis i IETF RFC 3066.
     */
    public void setXmlLang(String xmlLang){
        this.xmlLang = xmlLang;
    }
    
    /**Metoden legger til "ekstra" comment element. Kan gjentaes s ofte som nskelig.
     * @param comment   Kommentar element.
     */
    public void addComment(String comment){
        this.comment.add(comment);
    }
    
    /**
     * Metoden setter catalog element.
     *  @param catalog  Identifiserer standard vokabular for newsComponent. <br>
     *                  Den kan og indikerer hvor i NewsML dokumentet visse Topics er brukt. <br>
     *                  Elementet er frivillig, og kan kun forekomme 1 gang.<br>
     */
    public void setCatalog(Catalog catalog){
        this.catalog = catalog;
    }
    
    /**Metoden legger til "ekstra" topicSet element. Kan gjentaes s ofte som nskelig.
     * @param topicSet Beskriver emnet til nyhetskomponentet. <br><br>
     */
    public void addTopicSet(TopicSet topicSet){
        this.topicSet.add(topicSet);
    }
    
    
    /**Metoden legger til "ekstra" basisForChoice element. Kan gjentaes s ofte som nskelig.
     * @param basisForChoice  Hvis det finnes flere alternative elementer i attributtetet "equivalentsList", 
     *                        angir en de  komponentene her. Dette elementet bruker et Xpath mnster, eller en element-type navn 
     *                        for  identifisere den rette komponenten i forhold til de egenskapene som er gitt.<br><br>
     */
    public void addBasisForChoice(String basisForChoice){
        this.basisForChoice.add(basisForChoice);
    }
    
    /** Setter role elementet
     * @param role Spesifiserer rollen en nyhetskomponent spiller i den nyhetskomponenten den er den del av. <br><br>
     */
    public void setRole(String role){
        this.role = role;
    }
    
    /** Setter NewsLines  elementet
     * @param newsLines Angir karakteristiske egenskaper ved nyhetsobjektet. Disse er ment  tilfre en representasjon av metadata som er 
     *                  lett tilgjenglig for mennesker.<br><br>
     */
    public void setNewsLines(NewsLines newsLines){
        this.newsLines = newsLines;
    }
    
    /** Setter AdminsitrativMetadata  elementet
     * @param administrativeMetadata   Angir metadata som forteller om opprinnelsen til nyhetskomponenten.
     */
    public void setAdminMetadata(AdminMetadata administrativeMetadata){
        this.administrativeMetadata = administrativeMetadata;
    }
    
    /** Setter rightsMetadata elementet
     * @param rightsMetadata Angir metadata innholder informasjon om opphavsrett, og rett til bruk vedrrende nyhetskomponentene.
     *
     */
    public void setRightsMetadata(RightsMetadata rightsMetadata){
        this.rightsMetadata = rightsMetadata;
    }
    
    /** Setter DescriptiveMetadata elementet
     * @param descriptiveMetadata  Angir metadata innholder informasjon som beskriver innholdet til en nyhetskomponent.
     */
    public void setDescriptiveMetadata(DescriptiveMetadata descriptiveMetadata){
        this.descriptiveMetadata = descriptiveMetadata;
    }
    
    /** Setter metadata elementet
     * @param metadata  Angir metadata som ikke er dekket av de vrige. Gir metadata type navn.
     */
    public void setMetadata(String metadata){
        this.metadata = metadata;
    }
    
    /** Setter  metadataproperty elementet
     * @param metadataProperty  Angir metadata som ikke er dekket av de vrige. Gir metadata verdi.
     */
    public void setMetadataProperty(String metadataProperty){
        this.metadataProperty = metadataProperty;
    }
    
    /** legger til 1 NewsComponentSub element
     * @param newsComponentSub  Kaller et av elementene: NewsItem | NewsItemRef | NewsComponent | ContentItem  der kun et av de skal forekomme.
     */
    public void setNewsComponentSub(NewsComponentSub newsComponentSub){
        this.newsComponentSub = newsComponentSub;
    }
    
    /**
     * Metoden returnerer alle NewsComponent variablene, i xml-format.
     *
     */
    public String getNewsComponent(){
        String t ="<NewsComponent ";
        if (essential != null){ t = t + "Essential=\"" + essential + "\"";}
        if (equivalentsList != null){t = t + " EquivalentsList=\"" +equivalentsList + "\"";}
        t  =t + ">";
        if ( basisForChoice != null){
            Iterator i = basisForChoice.iterator();
            while (i.hasNext()){
                t = t + "<BasisForChoice>" + i.next().toString() + "</BasisForChoice>";
            }
        }
        if ( xmlLang != null){t = t + "<NewsComponent xml:lang=\"" + xmlLang + "\">";}
        if ( comment != null){
            Iterator i = comment.iterator();
            while (i.hasNext()) t = t + "<Comment>" + i.next().toString() + "</Comment>";
        }
        if ( catalog != null){t = t + catalog.getCatalog();}
        if ( topicSet != null){
            Iterator i = topicSet.iterator();
            while (i.hasNext())
                t = t + ((no.ingeberg.NewsML.TopicSet)i.next()).getTopicSet();
            
            //t = t + "<TopicSet FormalName=\"" + i.next().toString() + "\"/>";
        }
        if ( role != null) t = t + "<Role FormalName=\"" + role + "\"/>";
        if ( newsLines != null){t = t + newsLines.getNewsLines();}
        if ( administrativeMetadata != null){t = t + administrativeMetadata.getAdminMetadata();}
        if ( rightsMetadata != null){t = t + rightsMetadata.getRightsMetadata();}
        if ( descriptiveMetadata != null){t = t + descriptiveMetadata.getDescriptiveMetadata();}
        if ( metadata != null){
            t = t + "<Metadata> <MetadataType FormalName=\"" + metadata + "\"/>";
            t = t + "<Property FormalName=\"" + metadataProperty + "\"/>";
            t = t + "</Metadata>";
        }
        if ( newsComponentSub != null){t = t + newsComponentSub.getNewsComponentSub();}
        if ( xmlLang != null) {t =t + "</NewsComponent>";}
        t =t + "</NewsComponent>";
        return t;
    }
    
    public String toString(){
        return getNewsComponent();
    }
}