package no.ingeberg.newsmldb;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.io.*;

/**
 * @author Marit Ingeberg
 */
public class MyDerbyDB {
    private String framework = "embedded";
    private String driver = "org.apache.derby.jdbc.EmbeddedDriver";
    private String protocol = "jdbc:derby:";
    private String dbName = "derbyDB";
    private Connection conn = null;     // Blir satt i init()
    private Properties props = null;    // Blir satt i init()
    
    public MyDerbyDB() {
        try {
            init();
        } catch (Exception e) { /* IGNORE */ }
    }
    
    private void init() throws Exception {
        props = new Properties();
        props.put("user", "user1");
        props.put("password", "user1");
        
        // In an embedded environment, this will start up Derby, since it is not already running.
        Class.forName(driver).newInstance();
        // Oppretter databasen vist den ikkje finnast
        conn = DriverManager.getConnection(protocol + dbName +";create=true", props);
        conn.setAutoCommit(false);
        
        Statement st = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        boolean dataexist = false;
        boolean idexist = false;
        ResultSet result = st.executeQuery("SELECT TABLENAME FROM SYS.SYSTABLES");
        while (result.next()) {
            if (result.getString(1).equalsIgnoreCase("data")) dataexist = true;
            if (result.getString(1).equalsIgnoreCase("identifikasjonsId")) idexist = true;        
        }
        result.close();
        
        // Oppretter tabellen data om den ikkje finnast
        if (!dataexist) {
            // Oppretter SQL-script for  opprette ein tabell som kan holde p filer, som ein BLOB.
            StringBuffer createTable = new StringBuffer();
            createTable.append("CREATE TABLE data (");
            createTable.append("id INT primary key GENERATED ALWAYS AS IDENTITY,");
            createTable.append("description VARCHAR(50),");
            createTable.append("blob BLOB(2m),"); // 1mb er default blob strrelse
            createTable.append("filename VARCHAR(50),");
            createTable.append("filetype VARCHAR(15)");
            createTable.append(")");
            st.execute(createTable.toString());
            conn.commit();
        }
        
        // Oppretter tabellen identifikasjonsId om den ikke finnest
        if (!idexist) {
            // Oppretter SQL-script for  opprette ein tabell som kan holde p filer, som ein BLOB.
            StringBuffer createTableId = new StringBuffer();
            createTableId.append("CREATE TABLE identifikasjonsId(");
            createTableId.append("id INT primary key GENERATED ALWAYS AS IDENTITY,");
            createTableId.append("description VARCHAR(50)");
            createTableId.append(")");
            st.execute(createTableId.toString());
            conn.commit();
        }
        st.close();
    }
    
    public void close() {
        try { conn.close(); } catch (SQLException se) { /* ignore */ }
        try { DriverManager.getConnection("jdbc:derby:;shutdown=true"); } catch (SQLException se) { /* ignore */ }
    }
    
    /*
    public MyDerbyDB(boolean runTest) throws Exception {
        if (runTest == true) test();
    }
     
    public static void main(String[] args) throws Exception {
        new MyDerbyDB(true);
    }
     
    private void test() throws Exception {
        File file = new File("c:\\test.jpg");
        int pri = saveFile(file);
        System.out.println("Lagra fila " + file.getAbsolutePath() + " i basen p primaryKey = " + pri);
        file = new File("c:\\" + new Integer(pri).toString() + ".jpg");
        loadFile(pri,file);
        makeId();
        System.out.println("Lagra fila " + file.getAbsolutePath() + " til disk pga primaryKey = " + pri);
    }
     */
    
    /**
     * Henter ut en fil fra databasen og returnere den
     * @param int primaryKey Nkkelen til den nska fila i databasen
     * @param File file Target som fila i databasen skal lagrast til
     * @return File Den fila som er henta fra db, eller (File) null
     */
    public File loadFile(int primaryKey, File file) throws Exception {
        if (conn != null && props != null) {
            PreparedStatement ps;
            ps= conn.prepareStatement("SELECT blob FROM data WHERE id = ?");
            ps.setInt(1,primaryKey);
            ps.execute();
            ResultSet rs = ps.getResultSet();
            if (rs.next()) {
                Blob blob = rs.getBlob("blob");
                file.createNewFile();
                FileOutputStream out = new FileOutputStream(file);
                InputStream in = blob.getBinaryStream();
                int b;
                while ((b = in.read()) > -1)
                    out.write(b);
                out.flush();
                out.close();
                in.close();
            }
            ps.close();
            rs.close();
            return file;
        }
        return (File) null;
    }
    
    /**
     * Lagrer filen i databasen. Returnerer primrnkkel != -1 vist vellykka.
     * Vist databasen ikke finnest s blir den opprettet.
     */
    public int saveFile(File file) throws Exception{
        if (conn != null && props != null) {
            // Deler opp navnet p fila som vi skal lagre
            String ext = "";
            String pre = "";
            if (file.isFile() && file.getName().length() > 0 && file.getName().contains(".")) {
                pre = file.getName().substring(0,file.getName().lastIndexOf("."));
                ext = file.getName().substring(file.getName().lastIndexOf(".")+1, file.getName().length());
            }
            FileInputStream inputStream = new FileInputStream(file);
            PreparedStatement ps;
            ps= conn.prepareStatement("INSERT INTO data(blob, filename, filetype) VALUES (?,?,?)", Statement.RETURN_GENERATED_KEYS);
            ps.setBinaryStream(1,inputStream,(int) file.length());
            ps.setString(2,pre);
            ps.setString(3,ext);
            ps.execute();
            conn.commit();
            
            int primaryKey = 0;
            ResultSet rs = ps.getGeneratedKeys();
            if (rs.next()) primaryKey = rs.getInt(1);
            
            ps.close();
            rs.close();
            return primaryKey;
        }
        return -1;
    }
    
    /**
     * Returnerer en unik ubrukt id, klar til bruk.
     * Ved returverdi =0 er ikke id unik
     * Ved returverdi =-1 har kommunikasjon med databasen feilet
     */
    public int makeId(){
        int primaryKey = 0;
        if (conn != null && props != null) {
            
            try{
                PreparedStatement ps;
                ps= conn.prepareStatement("INSERT INTO identifikasjonsId(description)VALUES (?)", Statement.RETURN_GENERATED_KEYS);
                ps.setString(1,"test");
                ps.execute();
                conn.commit();
                
                ResultSet rs = ps.getGeneratedKeys();
                if (rs.next()) primaryKey = rs.getInt(1);
                
                ps.close();
                rs.close();
            } catch (Exception e) {
                //NOE ER GALT
                return -1;
            }
            return primaryKey;
        }
        return -1;
    }
}
